/***************************************************************************
 *   Copyright (C) 2007 by Christos Gkekas                                 *
 *   chggr@ee.auth.gr                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "hCommandHistory.h"

///-----------------------------------Constructor and Destructor-------------------------------------
/**
 *  @brief Class Constructor.
 *
 *  This is the hCommandHistory class constructor and it is called every time a new hCommandHistory
 *  object is created. Upon its creation, a hCommandHistory object must be initialized. This means
 *  that the variables "numDays" and "numSessions" must be equal to -1. Before the object can be used
 *  efficiently, other classes must utilize functions "setNumDays" or "setNumSessions" in order to
 *  alter the values of these two variables accordingly.
 */
hCommandHistory::hCommandHistory()
{
  numDays=-1;
  numSessions=-1;
  currentSessionCreated=false;
}


///--------------------------------------Basic Functions--------------------------------------------
/**
 *  @brief Sets the number of days.
 *
 *  This function is used to set the number of days history data will be kept. At first, it checks
 *  whether the provided integer value is logical. If the number provided is less than zero, no action
 *  should be taken. On the other hand, if the number provided is greater or equal to zero, private
 *  variable "numDays" is initialized.
 */
void hCommandHistory::setNumDays(int days)
{
  //At first check if the integer value provided as input is inside logical bounds (greater than or equal to zero). If it is not, return without doing anything.
  if (days < 0) return;

  //Check if both variables "numDays" and "numSessions" are equal to -1. This ensures that they have not been already initialized.
  if (numDays==-1 && numSessions==-1) numDays = days;
}

/**
 *  @brief Sets the number of sessions.
 *
 *  This function is used to set the number of sessions history data will be kept. At first, it checks
 *  whether the provided integer value is logical. If the number provided is less than zero, no action
 *  should be taken. On the other hand, if the number provided is greater or equal to zero, private
 *  variable "numSessions" is initialized.
 */
void hCommandHistory::setNumSessions(int sessions)
{
  //At first check if the integer value provided as input is inside logical bounds (greater than or equal to zero). If it is not, return without doing anything.
  if (sessions < 0) return;

  //Check if both variables "numDays" and "numSessions" are equal to -1. This ensures that they have not been already initialized.
  if (numDays==-1 && numSessions==-1) numSessions = sessions;
}

/**
 *  @brief Returns the contents of the history directory.
 *
 *  This function is used to return the contents of the history directory (~/.englab/history). This
 *  directory holds all history files and this method is used to read the names of the files, place
 *  them into a string vector, sort them and then return a string vector with the sorted filenames.
 *  If the directory does not exist, it has to be created.
 */
vector<string> hCommandHistory::readHistoryDirectory()
{
  //Create two string vectors.
  vector<string> filenames;
  vector<string> sortedFilenames;

  //Try to open the ~/.englab/history directory.
  string homeDir=getenv("HOME");  //Get the path to the home directory.
  string optionDir=homeDir+"/.englab";  //Add the path to the option directory.
  string historyDir=optionDir+"/history";  //Add the path to the history directory.
  DIR* directory = opendir(historyDir.c_str()); //Try to open the directory.

  //If the directory does not exist, create it and return.
  if (!directory)
  {
    string cmd="mkdir ";
    if((directory = opendir(optionDir.c_str()))==NULL)
    	system((cmd+optionDir).c_str());
    system((cmd+historyDir).c_str());
    return sortedFilenames;
  }

  //If the directory exists, read all filenames and store them inside vector "filenames".
  struct dirent* dirp;
  while ((dirp = readdir(directory)) != NULL) filenames.push_back(dirp->d_name);

  //Erase virtual files that have a filename with less than 14 digits
  for (unsigned int i=0; i<filenames.size(); i++)
  {
    if (filenames.at(i).length() < 14)
    {
      filenames.erase(filenames.begin()+i);
      i--;
    }
  }

  //If there are no files inside the history directory, return an empty string vector.
  if (filenames.size() == 0) return sortedFilenames;

  //If vector "filenames" is not empty, read it and sort the filenames. The final results will be placed inside vector "sortedFilenames".
  string temp;
  double max;
  int place;
  while (filenames.size() != 0)
  {
    //Get the first element of vector "filenames" and consider it as the maximum value.
    max = atof(filenames.at(0).data());
    place = 0;

    //Search the rest of vector "filenames" in order to find a value greater than max.
    for (unsigned int i=1; i<filenames.size(); i++)
    {
      //If a value greater than "max" is found, change the contents of "max" and "place" accordingly.
      if (atof(filenames.at(i).data()) > max)
      {
        max = atof(filenames.at(i).data());
        place = i;
      }
    }

    //Insert the maximum value inside vector "sortedFilenames" and erase it from vector "filenames".
    sortedFilenames.push_back(filenames.at(place));
    filenames.erase(filenames.begin()+place);
  }
  //Return vector "sortedFilenames".
  return sortedFilenames;
}

///------------------------------------------Other Functions------------------------------------------
/**
 *  @brief Deletes unnecessary files from the history directory.
 *
 *  This method is used to delete unnecessary files from the history directory. Unnecessary files are
 *  history files of older sessions or history files with filenames indicating that they were created
 *  before the specified "numDays" time period.
 */
void hCommandHistory::clearHistoryFiles()
{
  //Call function "readHistoryDirectory" to obtain a sorted list of all filenames.
  vector<string> filenames = readHistoryDirectory();

  //If the number of sessions criterion has been used, delete the files of older sessions.
  if (numSessions>-1)
  {
    string targetFile;
    for (unsigned int i=numSessions-1; i<filenames.size(); i++)
    {
      //Find the full path to the target file.
      targetFile = getenv("HOME");
      targetFile += "/.englab/history/";
      targetFile += filenames.at(i);

      //Delete the target file.
      system(("rm " + targetFile).c_str());
    }
  }

  if (numDays>-1)
  {
    //Find the "targetDayTime". Files with filenames indicating that they were created before this time will be deleted.
    string targetFile;
    string targetDayTime;
    time_t now = time(0);
    struct tm* localnow = localtime(&now);
    stringstream tempss;
    int tempint;
    tempss << localnow->tm_year + 1900;
    tempint = localnow->tm_mon + 1;
    if (tempint < 10) tempss << "0";
    tempss << tempint;
    tempint = localnow->tm_mday;
    if (tempint < 10) tempss << "0";
    tempss << tempint;
    tempint = localnow->tm_hour;
    if (tempint < 10) tempss << "0";
    tempss << tempint;
    tempint = localnow->tm_min;
    if (tempint < 10) tempss << "0";
    tempss << tempint;
    tempint = localnow->tm_sec;
    if (tempint < 10) tempss << "0";
    tempss << tempint;
    targetDayTime = tempss.str();

    //Check all files and delete all those created before the specified date.
    for (unsigned int i=0; i<filenames.size(); i++)
    {
      if (atof(filenames.at(i).data()) < atof(targetDayTime.data()))
      {
        //Find the name of the targe file
        targetFile = getenv("HOME");
        targetFile += "/.englab/history/";
        targetFile += filenames.at(i);

        //Erase the target file.
        system(("rm " + targetFile).c_str());
      }
    }
  }
}

   //Delete all history files.
void hCommandHistory::deleteHistory(){
	using namespace std;
	string path=hGlobals::optionsDir+"/history";
	system(("rm -Rf "+path).c_str());
}

/**
 *  @brief Reads all history data.
 *
 *  This function is used in order to read all history files and insert the information they contain
 *  inside vector historyData. All history files are located inside the ~/.englab/history directory and
 *  their filenames consist of the exact date and time each session has started. The files will be read
 *  with reverse chronological order.
 */
void hCommandHistory::readHistoryData()
{
  //Retrieve the names of the files inside "history" directory.
  vector<string> filenames = readHistoryDirectory();

  //Clear vector "historyData".
  historyData.clear();

  //Fill the contents of vector "historyData".
  string targetFile;
  string line;
  for (int i=filenames.size()-1; i>=0; i--)
  {
    //Open the file for reading.
    targetFile = getenv("HOME");
    targetFile += "/.englab/history/";
    targetFile += filenames.at(i);
    ifstream file(targetFile.c_str(), ios::in);

    //Read the contents of the file line by line and insert them into vector "historyData".
    while (!file.eof())
    {
      getline(file,line);
      historyData.push_back(line);
    }

    //Close the file.
    file.close();
  }
}

/**
 *  @brief Creates a new session.
 *
 *  Every time a new session is created, this function must be used. It retrieves the exact date and time
 *  and uses this information to generate the name of the new file which is going to be created in order
 *  to store the commands of this session. 
 */
void hCommandHistory::createCurrentSession()
{
  //Get current time.
  time_t now = time(0);
  struct tm* localnow = localtime(&now);

  //Find the name of the new history file which is going to be created.
  stringstream tempss;
  int tempint;
  tempss << localnow->tm_year + 1900;
  tempint = localnow->tm_mon + 1;
  if (tempint < 10) tempss << "0";
  tempss << tempint;
  tempint = localnow->tm_mday;
  if (tempint < 10) tempss << "0";
  tempss << tempint;
  tempint = localnow->tm_hour;
  if (tempint < 10) tempss << "0";
  tempss << tempint;
  tempint = localnow->tm_min;
  if (tempint < 10) tempss << "0";
  tempss << tempint;
  tempint = localnow->tm_sec;
  if (tempint < 10) tempss << "0";
  tempss << tempint;
  newFileName = tempss.str();

  //Initialize vector "currentSession".
  historyData.push_back("---> " + newFileName.substr(8,2) + ":" + newFileName.substr(10,2) + ":" + newFileName.substr(12,2) + "  -  " + newFileName.substr(6,2) + "/" + newFileName.substr(4,2) + "/" + newFileName.substr(0,4));

  //Set the boolean variable "currentSessionCreated" equal to true.
  currentSessionCreated = true;
}

/**
 *  @brief Adds commands to the history of a session.
 *
 *  When a new session is created, commands can be added to its history using this method. Every time
 *  the user types in a new command, the programm must use this function in order to register this
 *  command.
 */
void hCommandHistory::addCommand(string command)
{
  if (command != "" && currentSessionCreated) historyData.push_back(command);
}

/**
 *  @brief Saves the history of a session in a text file.
 *
 *  This method is used in order to save the commands of a session into a new history file. The name
 *  of the file has already been generated by function "createCurrentSession". At first, the value of
 *  the flag "currentSessionCreated" is checked to determine whether a new session has already been
 *  created or not. In the second case, no action is taken.
 */
void hCommandHistory::saveCurrentSession()
{
  //If the user has not created the current session, simply return without doing anything.
  if (!currentSessionCreated) return;

  //Locate the last session entry inside vector "historyData".
  unsigned int marker = historyData.size()-1;
  for (; marker>=0; marker--)
  {
    if (historyData.at(marker).substr(0,4) == "--->") break;
  }

  //Save the session in a file named as "newFileName" under ~/.englab/history/ directory.
  string targetFile;
  targetFile = getenv("HOME");
  targetFile += "/.englab/history/";
  targetFile += newFileName;

  //Open the file for writing and record all commands.
  ofstream file (targetFile.c_str());
  for (; marker<historyData.size(); marker++)
  {
    file << historyData.at(marker);
    if (marker < historyData.size()) file << endl;
  }

  //Finally, close the file.
  file.close();
}

/**
 *  @brief Returns the history data.
 *
 *  This method returns a string vector containing all history retrieved from already existing history
 *  files plus all commands the user has generated in this session.
 */
vector<string> hCommandHistory::getHistoryData()
{
  return historyData;
}

/**
 *  @brief Returns the history data.
 *
 *  This method is used to print to the standard output all history data obtained by now.
 */
void hCommandHistory::printHistoryData()
{
  //At first print an initial message.
  cout << endl << " COMMAND HISTORY" << endl;

  //Display all commands and a counter for counting them.
  int counter = 1;
  for (unsigned int i=0; i<historyData.size(); i++)
  {
    //When a timestamp is displayed, do not increase the counter.
    if (historyData.at(i).substr(0,4) == "--->")
    {
      cout << endl << historyData.at(i);
    }
    else
    {
      //Display and increase the counter value only when a real command is encountered.
      cout << endl << counter++ << "  " << historyData.at(i);
    }
  }

  //Leave two empty lines.
  cout << endl << endl;
}

/**
 *  @brief Returns a specific command
 *
 *  This method is used return a specific command from the history. The exact location of the desired
 *  command must be given a input. The program searches 
 */
string hCommandHistory::getHistoryCommand(int location)
{
  //Initialize the counter.
  int counter = 0;

  //Traverse vector historyData in order to find the requested command.
  for (unsigned int i=0; i<historyData.size(); i++)
  {
    if (historyData.at(i).substr(0,4) != "--->")
    {
      //Increase the counter value.
      counter ++;

      //Once the command is found, return it.
      if (counter == location) return historyData.at(i);
    }
  }
  return "";
}

