/***************************************************************************
 *   Copyright (C) 2007 by Harry Serenis                                   *
 *   bobomastoras@users.sf.net                                             * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HVARIABLEHEADERS_H
#define HVARIABLEHEADERS_H

#include <complex>
//namespace oLab_Variable{

///Stores variables/tables of different types, dimensions and sizes
/**hVariable supports variables with types:<br><br>
unsigned char<br>signed char<br>unsigned short int<br>signed short int<br>unsigned long int<br>signed long int<br>float<br>double<br>long double<br>complex<float><br>complex<double><br>complex<long double><br>
hVariable actually uses hVarTemplate as a back-end but handles all memmory allocations and variable's dataTypes. Methods and operators can be used without the knowledge of the actual template instantization for hVarTemplate and operators can be called for variables with different data types*/

class hVariable{
	void *var;
	unsigned char dataType;
	template <class type>
	void  setType(type *data) throw();
	void setType(unsigned char dType);
// 	bool readOnly;
	hVariable addConst(const hVariable op2);
	hVariable subConst(const hVariable op2);
	hVariable mulConst(const hVariable op2);
	hVariable divConst(const hVariable op2);
public:
	hVariable cast(unsigned char type);
///A simple constructor
	hVariable(void) throw();
	static std::string literalType(unsigned char t);

///Allocates the required memmory and stores a variable/table
/**All variables are treated as tables of at least 2 dimensions. A simple variable is a table of size 1x1 
\param dim is the number of table's dimensions

\param size is a table of at least dim elements. Each element is the size of the corresponding dimension
\param data is a single dimensional table that holds up the values needed to be stored by hVariable. Table data must have \prod_{i=1}^dimensions\size(i) elements. Values are inoperant1dexed dy dimension.
e.g. In order to store table:
<table align="center" border="1"  cellspacing="1" cellpadding="0">

<tr><td>4</td><td>1</td><td>6</td></tr><tr><td>2</td><td>6</td><td>1</td></tr><tr><td>7</td><td>5</td><td>3</td></tr></table> values={4,1,6,2,6,1,7,5,3}*/
	template <class type>
	hVariable(unsigned int dim,unsigned int *size,type *data) throw(std::exception);

///Copy constructor
/**If the = operator is used to copy an hVariable object the copy constructor is called. The result is a fake copy of the original object, that points to the same allocated space. Therefore the variable is marked as "read only". In order to make an actual copy use hVariable::copy(void)*/
// 	hVariable(const hVariable& p);

///Set the value of an element
/**Set the value of the element indexed by parameter index, equal to data
\param index is a table containing at least dimensions elemets. Each element indexes the corresponding dimension. e.g. In a 3 dimensional table, to index element (2,4,1), index = {2,4,1} 
\param data is a variable of type dataType. The indexed element is set equal to data*/
	void setValue(unsigned int* index,hVariable data);

///Returns the value of the indexed element
/**The element is indexed by parameter index.
\param index is a table containing at least dimensions elements. Each element indexes the corresponding dimension. e.g. In a 3 dimensional table, to index element (2,4,1), index = {2,4,1}
\returns the stored value in element indicated by index*/
	template <class type>

	type getValue(unsigned int *index) throw(std::exception);

///Returns variable's/table's type
/**The returned value is an unsigned char indicating the type of the variable. All types and their meanings can be found in file hVarType.h
\returns the type of data stored. Data types are defined int file hVarType.h*/
	unsigned char getType(void) throw();

///Deletes allocated memory
/**Calls hVarTemplate::erase*/
	void erase(void);

/**@name Operators Overloading
All C arithmetic, boolean and binary operators are overloaded for class hVariable. In case that the operator cannot be implemented, an hException is thrown. If the operator is implemented on variables with different types, the variable with a lower priority type is first casted to the type of the other operant. For type priorities see file hVarType.h*/
///@{
	hVariable operator=(const hVariable &op2);
	hVariable operator+(hVariable op2);
	hVariable operator-(hVariable op2);
	hVariable operator*(hVariable op2);
	hVariable operator/(hVariable op2);
	hVariable operator||(hVariable op2);
	hVariable operator&&(hVariable op2);
	hVariable operator!(void);



	hVariable operator==(hVariable op2);
	hVariable operator!=(hVariable op2);
	hVariable operator>=(hVariable op2);
	hVariable operator<=(hVariable op2);
	hVariable operator<(hVariable op2);
	hVariable operator>(hVariable op2);
	hVariable operator|(hVariable op2);
	hVariable operator&(hVariable op2);
	hVariable operator^(hVariable op2);

	template <class type>
	hVariable operator+(type op2);
	template <class type>
	hVariable operator-(type op2);
	template <class type>
	hVariable operator*(type op2);
	template <class type>
	hVariable operator/(type op2);

///@}
///Returns the number of table's dimensions
/** \returns the number of dimensions table has*/
	unsigned int getDim(void) const;

///Returns the size of every dimension.
/**The returned value is a pointer to a table with dimensions elements. Each element of the table is the size of the corresponding dimension
\returns a c table that contains the size of every dimension*/
	unsigned int *getSize(void);

///Returns a copy of the object
	hVariable copy(void);

/// << Operator overloading for ostreams
/** << \Operator is overloaded in order to easily output a Variable/Table to console.*/
	friend std::ostream& operator << (std::ostream& output, const hVariable& s);

///Cast table to type
/**Returns a hVariable object of the same size and type type. The elements of the returned object are set equal to the corresponding original elements after they are casted to type.*/
	template <class type>
	hVariable convert(void) const;

	template <class type>
	type get(void) const;
	
	hVariable gethVariable(unsigned int *index);

	hVariable traverse(unsigned char (*function)(unsigned char));
	hVariable traverse(signed char (*function)(signed char));
	hVariable traverse(unsigned short int (*function)(unsigned short int));
	hVariable traverse(signed short int (*function)(signed short int));
	hVariable traverse(unsigned long int (*function)(unsigned long int));
	hVariable traverse(signed long int (*function)(signed long int));
	hVariable traverse(float (*function)(float));
	hVariable traverse(double (*function)(double));
	hVariable traverse(long double (*function)(long double));
	hVariable traverse(std::complex<float> (*function)(const std::complex<float> &));
	hVariable traverse(std::complex<double> (*function)(const std::complex<double> &));
	hVariable traverse(std::complex<long double> (*function)(const std::complex<long double> &));

};

#endif
